# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

try:
    # flake8 2.x
    from flake8 import engine
except ImportError:
    # flake8 3.x
    from flake8.api import legacy as engine

import re
import tempfile

import hacking.tests


def check(physical_line):
    """Test check to make sure local-checks are working."""
    if physical_line.strip() == "#this-is-the-test-phrase":
        return (0, "L100: Found local-check test case")


@testtools.skip("local-checks is broken with flake8 3.x")
class HackingTestCase(hacking.tests.TestCase):
    def test_local_check(self):
        flake8_style = engine.get_style_guide(parse_argv=False, ignore='F')
        with tempfile.NamedTemporaryFile() as tempfh:
            tempfh.write("#this-is-the-test-phrase\n")
            tempfh.flush()
            report = flake8_style.check_files([tempfh.name])

        stats = report.get_statistics('')
        codes = set()
        for stat in stats:
            codes.update(re.findall(r'[A-Z]\d{3,4}', stat))

        self.assertIn("L100", codes)
